@@ -1,10 +1,10 @@
 package PAR::Packer;
-$PAR::Packer::VERSION = '0.969_01';
-
 use 5.006;
 use strict;
 use warnings;
 
+our $VERSION = '0.989_01';
+
 =head1 NAME
 
 PAR::Packer - PAR Packager
@@ -44,6 +44,7 @@ use constant OPTIONS => {
     'c|compile'      => 'Compile code to get dependencies',
     'd|dependent'    => 'Do not include libperl',
     'e|eval:s'       => 'Packing one-liner',
+    'E|evalfeature:s'=> 'Packing one-liner with new syntactic features',
     'x|execute'      => 'Execute code to get dependencies',
     'X|exclude:s@'   => 'Exclude modules',
     'f|filter:s@'    => 'Input filters for scripts',
@@ -86,14 +87,6 @@ sub new {
     # exit gracefully and clean up after ourselves.
     # note.. in constructor because of conflict.
 
-    # Up to PAR 0.957, we set the following ENV variable, but
-    # it is never, ever used anywhere in the rest of the
-    # PAR or M::SD sources, so in order to remove clutter, we
-    # comment it out. This notice and the following commented
-    # out line should be removed in a future version if no
-    # breakage occurred.
-
-    # $ENV{PAR_RUN} = 1;
     my $self = bless {}, $class;
 
     $self->set_args($args)      if ($args);
@@ -163,6 +156,7 @@ sub set_front {
     $self->{frontend} = $frontend || $opt->{frontend};
 }
 
+# check one or more files for read permissions
 sub _check_read {
     my ($self, @files) = @_;
 
@@ -184,6 +178,7 @@ sub _check_read {
     }
 }
 
+# check one or more files for write permissions
 sub _check_write {
     my ($self, @files) = @_;
 
@@ -200,6 +195,7 @@ sub _check_write {
     }
 }
 
+# check whether a given file might contain perl code (including .par's)
 sub _check_perl {
     my ($self, $file) = @_;
     return if ($self->_check_par($file));
@@ -301,6 +297,12 @@ sub _parse_opts {
     $self->{logfh} = $self->_open('>>', $opt->{L})
       if length $opt->{L};
 
+    if ($opt->{E}) {
+        $opt->{e} = "use $];\n#line 1\n$opt->{E}";
+        # XXX This is how we should also include additional default modules in the future instead of in require_modules in par.pl!
+        push @{$opt->{M}||=[]}, 'feature' if $] >= 5.009;
+    }
+
     if ($opt->{e}) {
         $self->_warn("Using -e 'code' as input file, ignoring @$args\n")
           if (@$args and !$opt->{r});
@@ -353,6 +355,7 @@ sub _create_valid_hash {
     }
 }
 
+# prints a dump of the OPTIONS hash
 sub _show_usage {
     my ($self) = @_;
 
@@ -425,7 +428,9 @@ sub run_pack {
     my $output = $self->{output};
     my $args   = $self->{args};
 
-    $output = File::Spec->catfile(".", $output);
+    if (!File::Spec->file_name_is_absolute($output)) {
+	$output = File::Spec->catfile(".", $output);
+    }
 
     my @loader = ();
     push(@loader, $^X) if ($opt->{P});
@@ -574,7 +579,7 @@ YAML
         $dep_list = join ', ', keys %dep_zips;
         $self->_vprint(0, "$self->{output} will require $dep_list at runtime");
 
-        $manifest =~ s/$_\n// for (keys %dep_zip_files);
+        $manifest =~ s/\Q$_\E\n// for (keys %dep_zip_files);
     }
 
     $self->_vprint(2, "... updating $_") for qw(MANIFEST META.yml);
@@ -661,6 +666,9 @@ sub pack_manifest_hash {
         $self->_name2moddata($name, \@modules, \@data);
     }
 
+    # Skip either
+    # a) all files from a .par file or
+    # b) A module
     foreach my $name ('PAR', @{ $opt->{X} || [] }) {
         if (-f $name and my $dep_zip = Archive::Zip->new($name)) {
             for ($dep_zip->memberNames()) {
@@ -694,7 +702,8 @@ sub pack_manifest_hash {
     # Apply %Preload to the -M'd modules and add them to the list of
     # files to scan
     foreach my $module (@modules) {
-        my $file = &$inc_find($module);
+        my $file = &$inc_find($module)
+          or $self->_die("Cannot find module $module (specified with -M)\n");
         push @files, $file;
         
         my $preload = Module::ScanDeps::_get_preload($module) or next;
@@ -1243,13 +1252,9 @@ sub _par_to_exe {
         $self->_append_parl();
         $self->_generate_output();
 
-        Win32::Exe->new($output)->update(
-            icon => $opt->{i},
-            info => $opt->{N},
-        );
-
         $self->_fix_console();
         unlink($self->{parl});
+        unlink($self->{orig_parl});
         unlink("$self->{parl}.bak");
         return;
     }
@@ -1364,8 +1369,11 @@ sub _generate_output {
     my $output   = $self->{output};
     my $par_file = $self->{par_file};
 
-    my @args = ('-B', "-O$output", $par_file);
+    my @args = ("-O$output", $par_file);
     unshift @args, '-q' unless $opt->{v} > 0;
+    if ($opt->{B}) {
+        unshift @args, "-B";
+    } 
     if ($opt->{L}) {
         unshift @args, "-L".$opt->{L};
     }
@@ -1669,7 +1677,7 @@ Please submit bug reports to E<lt>bug-par@rt.cpan.orgE<gt>.
 
 =head1 COPYRIGHT
 
-Copyright 2004, 2005, 2006 by Audrey Tang E<lt>cpan@audreyt.orgE<gt>.
+Copyright 2004-2008 by Audrey Tang E<lt>cpan@audreyt.orgE<gt>.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
